@@ -45,6 +45,10 @@
 #define STACK_DIRECTION (-1)
 #endif /*0*/
 
+#include <bitypes.h>
+#define INT32_MAX __INT32_MAX
+#define INT32_MIN __INT32_MIN
+
 /* dynamic loading is possible */
 #undef DYNAMIC
 
@@ -399,7 +403,7 @@
 #define PACKAGE_NAME "GNU Awk"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "GNU Awk 4.0.2"
+#define PACKAGE_STRING "GNU Awk 4.1.0"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "gawk"
@@ -408,7 +412,7 @@
 #undef PACKAGE_URL
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "4.0.2"
+#define PACKAGE_VERSION "4.1.0"
 
 /* Define to 1 if *printf supports %F format */
 #undef PRINTF_HAS_F_FORMAT
@@ -438,7 +442,7 @@
 #define USE_INCLUDED_STRFTIME 1
 
 /* Version number of package */
-#define VERSION "4.0.2"
+#define VERSION "4.1.0"
 
 /* Define to 1 if on AIX 3.
    System headers sometimes define this.
@@ -545,6 +549,10 @@
 /* Whether `time_t' is an unsigned type. */
 #define TIME_T_UNSIGNED 1
 
+/* Extension for shared libraries */
+#define SHLIBEXT	"exe"
+
+
 /*******************************/
 /* Gawk configuration options. */
 /*******************************/
@@ -561,6 +569,7 @@
  */
 
 #define DEFPATH	".,/AWK_LIBRARY"
+#define DEFLIBPATH ".,/AWK_LIBRARY"
 #define ENVSEP	','
 
 /*
